//+------------------------------------------------------------------+
//|                                                         EMMA.mq4 |
//|                                  Extremums Market Moving Average |
//|                               Copyright  2010,   |
//|                                            mql.sergeev@yandex.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010,  "
#property link      "mql.sergeev@yandex.ru"

#property indicator_chart_window
#property indicator_buffers 1
#property indicator_color3 Yellow
//+------------------------------------------------------------------+
extern double EMMA_Deviation_in_Percent = 0.1;       //   ,   .
//+------------------------------------------------------------------+
extern int EMMA_Mode = 0;     //  
                              //PRICE_CLOSE    0     
                              //PRICE_HIGH/LOW 1     ./.
//+------------------------------------------------------------------+
extern bool EMMA_Redraw = true;    //./. 
//+------------------------------------------------------------------+
extern int MA_Period = 18;    //     .
//+------------------------------------------------------------------+
extern int MA_Method = 0;     //   .         (Moving Average):
                              /*MODE_SMA       0      
                                MODE_EMA       1      
                                MODE_SMMA      2      
                                MODE_LWMA      3   -   */           
//+------------------------------------------------------------------+
extern int MA_Applied_Price = 0; //   .      :
                              /*PRICE_CLOSE    0     
                                PRICE_OPEN     1     
                                PRICE_HIGH     2     
                                PRICE_LOW      3     
                                PRICE_MEDIAN   4    , (high+low)/2 
                                PRICE_TYPICAL  5    , (high+low+close)/3 
                                PRICE_WEIGHTED 6     , (high+low+close+close)/4 */
//+------------------------------------------------------------------+
extern bool Manager_Murrey = true;        //  Murrey_Math_Line v2
//+------------------------------------------------------------------+
extern bool Signal = false;               //./.  
//+------------------------------------------------------------------+
extern bool Fibo = false;                 //./.  .
//+------------------------------------------------------------------+
extern bool Fibo_Fan = false;             //./.  .
//+------------------------------------------------------------------+
extern bool Regression_Channel = false;   //./.   .
//+------------------------------------------------------------------+
double EMMABuffer[];                      // 
double ima;
datetime home_Time;
int position;
int global_shift;
//+------------------------------------------------------------------+
//| EMMA initialization function                                     |
//+------------------------------------------------------------------+
int init(){
   IndicatorBuffers(1);
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS));
   SetIndexStyle(0,DRAW_SECTION);
   SetIndexBuffer(0,EMMABuffer);
   SetIndexEmptyValue(0,0.0);
   IndicatorShortName("EMMA("+DoubleToStr(EMMA_Deviation_in_Percent, 4)+","+MA_Period+")");
   if(MA_Applied_Price > 6 || MA_Applied_Price < 0)MA_Applied_Price = 0;      //
   if(EMMA_Mode > 1 || EMMA_Mode < 0)EMMA_Mode = 0;                           //     
   if(MA_Method > 3 || MA_Method < 0)MA_Method = 0;                           //
   if(MA_Period < 2)MA_Period = 18;                                           //
   EMMA_Deviation_in_Percent /= 100;
}
//+------------------------------------------------------------------+
//| EMMA                                                             |
//+------------------------------------------------------------------+
int start(){
   static int ctrl = -1;
   static bool flag;
   static double save_Price;
   if(Bars-IndicatorCounted()-1 > 1)ArrayInitialize(EMMABuffer,0.0);          // "" ,  ,    
   for(int shift = Bars-IndicatorCounted()-1; shift >= 1; shift--){           //   
      global_shift = shift;
      ima = iMA(NULL, 0, MA_Period, 0, MA_Method, MA_Applied_Price, global_shift);
//+-------------------------------------------------------------------------------------------+
      if(ctrl != 0 && !flag && getTop() != -1){                      //   
         save_Price = getTop();
         flag = true;
         ctrl = 0;
      }
      if(ctrl == 0 && flag && ima <= save_Price - save_Price * EMMA_Deviation_in_Percent){ //   
         search(0);                                                //  
         flag = false;
         position = 1;           //top
      }
      if(ctrl == 0 && flag && ima >= save_Price){                         // 
         flag = false;
         ctrl = 1;
      }
//+-------------------------------------------------------------------------------------------+
      if(ctrl != 1 && !flag && getBottom() != -1){                   //   
         save_Price = getBottom();
         flag = true;
         ctrl = 1;
      }
      if(ctrl == 1 && flag && ima >= save_Price + save_Price * EMMA_Deviation_in_Percent){ //   
         search(1);                                                //  
         flag = false;
         position = 2;           //bottom
      }
      if(ctrl == 1 && flag && ima <= save_Price){                         // 
         flag = false;
         ctrl = 0;
      }
//+-------------------------------------------------------------------------------------------+
      reDraw();                  //redraw
      manageMurrey();            //manager
      signal();                  //alert
      fibo();                    //fibonacci
      fiboFan();                 //fibonacci fan
      regressionChannel();       //regression channal
   }
}
//+------------------------------------------------------------------+
//| Search extremums                                                 |
//+------------------------------------------------------------------+
void search(int type, bool write = true){
   static bool flag;
   int searchShift;
   if(type == 0){
      switch(EMMA_Mode){
         case 0:
            searchShift = iHighest(NULL, 0, MODE_CLOSE, iBarShift(NULL, 0, home_Time)-global_shift, global_shift);
            EMMABuffer[searchShift] = Close[searchShift];
         break;
         case 1:
            searchShift = iHighest(NULL, 0, MODE_HIGH, iBarShift(NULL, 0, home_Time)-global_shift, global_shift);
            EMMABuffer[searchShift] = High[searchShift];
         break;
      }
   }else{
      switch(EMMA_Mode){
         case 0:
            searchShift = iLowest(NULL, 0, MODE_CLOSE, iBarShift(NULL, 0, home_Time)-global_shift, global_shift);
            EMMABuffer[searchShift] = Close[searchShift];
         break;
         case 1:
            searchShift = iLowest(NULL, 0, MODE_LOW, iBarShift(NULL, 0, home_Time)-global_shift, global_shift);
            EMMABuffer[searchShift] = Low[searchShift];
         break;
      }
   }
   if(write)home_Time = Time[searchShift];
}
//+------------------------------------------------------------------+
//| Search top                                                       |
//+------------------------------------------------------------------+
double getTop(){
   double previous = iMA(NULL, 0, MA_Period, 0, MA_Method, MA_Applied_Price, global_shift + 1);
   if(ima < previous)return(previous);
   return(-1);
}
//+------------------------------------------------------------------+
//| Search bottom                                                    |
//+------------------------------------------------------------------+
double getBottom(){
   double previous = iMA(NULL, 0, MA_Period, 0, MA_Method, MA_Applied_Price, global_shift + 1);
   if(ima > previous)return(previous);
   return(-1);
}
//+------------------------------------------------------------------+
//| Manager Murrey                                                   |
//+------------------------------------------------------------------+
void manageMurrey(){
   if(ObjectFind("MML_StartBar") != -1 && Manager_Murrey){
      ObjectSet("MML_StartBar", OBJPROP_TIME1, getTime());
      ObjectSet("MML_StartBar", OBJPROP_PRICE1, getDate());
      WindowRedraw();
   }
}
//+------------------------------------------------------------------+
//| Alert                                                            |
//+------------------------------------------------------------------+
void signal(){
   static double price;
   if(!Signal)return;
   if(position == 1 && price != getDate()){
      if(EMMA_Redraw)Alert(Symbol(),"  .");else Alert(Symbol(),"  .");
   }
   if(position == 2 && price != getDate()){
      if(EMMA_Redraw)Alert(Symbol(),"  .");else Alert(Symbol(),"  .");
   }
   price = getDate();
}
//+------------------------------------------------------------------+
//| Redraw                                                           |
//+------------------------------------------------------------------+
void reDraw(){
   if(!EMMA_Redraw)return;
   if(!iBarShift(NULL, 0, home_Time)-1 > global_shift)return;
   for(int loop = iBarShift(NULL, 0, home_Time)-1; loop > global_shift; loop--)EMMABuffer[loop] = 0.0;
   if(position == 2)search(0, false);
   if(position == 1)search(1, false);
}
//+------------------------------------------------------------------+
//| getDate                                                          |
//+------------------------------------------------------------------+
double getDate(bool next = false){
   for(int loop = global_shift;loop < Bars;loop++){
      if(EMMABuffer[loop] != 0){
         if(next){
            next = false;
            continue;
         }
         return(EMMABuffer[loop]);
      }
   }
}
//+------------------------------------------------------------------+
//| getTime                                                          |
//+------------------------------------------------------------------+
datetime getTime(bool next = false){
   for(int loop = global_shift;loop < Bars;loop++){
      if(EMMABuffer[loop] != 0){
         if(next){
            next = false;
            continue;
         }
         return(Time[loop]);
      }
   }
}
//+------------------------------------------------------------------+
//| Fibo                                                             |
//+------------------------------------------------------------------+
void fibo(){
   if(!Fibo)return;
   if(ObjectFind("EMMA_FIBO") != -1){
      ObjectSet("EMMA_FIBO", OBJPROP_TIME1, getTime(true));
      ObjectSet("EMMA_FIBO", OBJPROP_PRICE1, getDate(true));
      ObjectSet("EMMA_FIBO", OBJPROP_TIME2, getTime());
      ObjectSet("EMMA_FIBO", OBJPROP_PRICE2, getDate());
      WindowRedraw();
   }else{      
      ObjectCreate("EMMA_FIBO", OBJ_FIBO, 0, getTime(true), getDate(true), getTime(), getDate());
      ObjectSet("EMMA_FIBO", OBJPROP_COLOR, Black);
      ObjectSet("EMMA_FIBO", OBJPROP_LEVELCOLOR, Black);
   }
}
//+------------------------------------------------------------------+
//| Fibo Fan                                                         |
//+------------------------------------------------------------------+
void fiboFan(){
   if(!Fibo_Fan)return;
   if(ObjectFind("EMMA_FIBO_FAN") != -1){
      ObjectSet("EMMA_FIBO_FAN", OBJPROP_TIME1, getTime(true));
      ObjectSet("EMMA_FIBO_FAN", OBJPROP_PRICE1, getDate(true));
      ObjectSet("EMMA_FIBO_FAN", OBJPROP_TIME2, getTime());
      ObjectSet("EMMA_FIBO_FAN", OBJPROP_PRICE2, getDate());
      WindowRedraw();
   }else{      
      ObjectCreate("EMMA_FIBO_FAN", OBJ_FIBOFAN, 0, getTime(true), getDate(true), getTime(), getDate());
      ObjectSet("EMMA_FIBO_FAN", OBJPROP_COLOR, Black);
      ObjectSet("EMMA_FIBO_FAN", OBJPROP_LEVELCOLOR, Black);
   }
}
//+------------------------------------------------------------------+
//| Regression Channel                                               |
//+------------------------------------------------------------------+
void regressionChannel(){
   if(!Regression_Channel)return;
   if(ObjectFind("EMMA_Regression_Channel") != -1){
      ObjectSet("EMMA_Regression_Channel", OBJPROP_TIME1, getTime(true));
      ObjectSet("EMMA_Regression_Channel", OBJPROP_PRICE1, getDate(true));
      ObjectSet("EMMA_Regression_Channel", OBJPROP_TIME2, getTime());
      ObjectSet("EMMA_Regression_Channel", OBJPROP_PRICE2, getDate());
      WindowRedraw();
   }else{      
      ObjectCreate("EMMA_Regression_Channel", OBJ_REGRESSION, 0, getTime(true), getDate(true), getTime(), getDate());
      ObjectSet("EMMA_Regression_Channel", OBJPROP_COLOR, Black);
   }
}
//+------------------------------------------------------------------+
//| EMMA deinitialization function                                   |
//+------------------------------------------------------------------+
int deinit(){
   if(ObjectFind("EMMA_FIBO") != -1)ObjectDelete("EMMA_FIBO");
   if(ObjectFind("EMMA_FIBO_FAN") != -1)ObjectDelete("EMMA_FIBO_FAN");
   if(ObjectFind("EMMA_Regression_Channel") != -1)ObjectDelete("EMMA_Regression_Channel");   
}
//+------------------------------------------------------------------+